REBOL [
  Title:  "OSR/miasta" 
  Version: 1.3.1
  Date:    2005-02-25
  Author: "Piotr Gapinski"
  Email:  {news [at] rowery! olsztyn.pl}
  File:   %osr-encoding.r
  Info: {
    Program upraszczajacy zbieranie informacji o miastach (na potrzeby OSR).
    Olsztynska Strona Rowerowa http://www.rowery.olsztyn.pl/
    PROGRAM NA LICENCJI GPL (w wersji II)
  }
]

osr-encoding-ctx: context [
  standards: [
    ; tabela standardow zamiany 18 polskich znakow (duze/male)
    windows-1250 [165 198 202 163 209 211 140 143 175 185 230 234 179 241 243 156 159 191]
    iso-8859-2   [161 198 202 163 209 211 166 172 175 177 230 234 179 241 243 182 188 191]
    amiga-pl     [194 202 203 206 207 211 212 218 219 226 234 235 238 239 243 244 250 251]
    iso-8859-1   [065 067 069 076 078 079 083 090 090 097 099 101 108 110 111 115 122 122] ;; "ASCII" 
  ]

  set 'rebol-encoding does [
    any [
      select [1 {amiga-pl} 3 {windows-1250} 4 {iso-8859-1}] fourth system/version
     {iso-8859-1}
    ]
  ]

  set 'plchars func [
   "Konwertuje polskie znaki w tekscie; Zwraca _kopie_ tekstu po konwersji"
    txt [string!] "tekst do konwersji"
    inp [string!] "nazwa standardu wejsciowego"
    out [string!] "nazwa standardu wyjsciowego"
   /local i c here] [

    if equal? lowercase inp lowercase out [return txt]

    inp: select standards to-word lowercase inp ; [block!] "wejsciowa tablica konwersji"
    out: select standards to-word lowercase out ; [block!] "wyjsciowa tablica konwersji"
    if none? all [inp out] [return txt]

    parse/all copy txt [
      any [
        here: skip ( 
          if 127 < c: first here [
            any [
              none? i: attempt [index? find inp to-integer c]
           ;; none? print ["c=" c "i=" i "o=" to-char out/:i]
              change here to-char out/:i
            ]
          ]
        ) :here
        skip
      ]
    ]
    head here
  ]

  tokens: ["amp" {&} "lt" {<} "gt" {>} "nbsp" { } "apos" {'} "quot" {"} "raquo" {-}]
  escape-html: func [
   "Zamienia encje HTML na tekst; Zwraca kopie tekstu po konwersji"
    txt [string!] "tekst zawierajacy encje HTML"
   /local here there entity] [

    entity: complement charset { :;<>&#}
    parse/all copy text [
      any [ 
        here: 
        end break 
        | "&" 
          [
            "#" copy item to ";" skip there: (
                  remove/part here there 
                  attempt [insert here form to-char to-integer item]
                ) 
                :here

             |  copy item some entity ";" there: (
                  code: select tokens item
                  if not none? code [
                    remove/part here there
                    insert here code
                    there: here
                  ]
                ) 
                :there
          ]
        | ["<![" "CDATA[" | "]]>"] there: (remove/part here there) :here
        | skip
      ] 
    ]
    head here
  ]
  
  set 'strip-html func [
   "Usuwa znaczniki HTML z tekstu; Zwraca tekst po konwersji"
    txt [string!] "tekst zawierajacy encje HTML"
   /local page] [

    remove-each tag page: load/markup escape-html txt [tag? tag]
    form page
  ]
]
