REBOL [
  Title:  "OSR/miasta" 
  Version: 1.3.1
  Date:    2005-02-25
  Author: "Piotr Gapinski"
  Email:  {news [at] rowery! olsztyn.pl}
  File:   %osr-file.r
  Info: {
    Program upraszczajacy zbieranie informacji o miastach (na potrzeby OSR).
    Olsztynska Strona Rowerowa http://www.rowery.olsztyn.pl/
    PROGRAM NA LICENCJI GPL (w wersji II)
  }
]

osr-file-ctx: context [
  ; zaleznosci z innymi plikami projektu
  ;
  do %osr-encoding.r
  do %xml/xml-parse.r
  do %xml/xml-object.r

  set 'city-object make object! [
    name: [{} {}]    ; [aktualna historyczna]
    direction: {}    ; [polozenie]
    gps: [0 0 0]     ; [latitude longitude elevation]
    info: {}         ; [opis]
    tip: []          ; [tip tip tip]
    param: []        ; [param param param]
  ]

  set 'osr-save func [
   "Zapisuje informacje o miejscowosci do pliku XML (bez weryfikacji DTD); Zwraca true! lub none!"
    city [object!] "informacje o miejscowosci"
    file [string! file!] "nazwa pliku"
    author [string!] "informacje o autorze"
    email [email! string!] "adres email"
   /force "wymuszenie nadpisania pliku"
   /local ciekawostki parametry
  ][
  
    ciekawostki: copy {}  parametry: copy {}
    foreach tip city/tip [append ciekawostki rejoin [form <ciekawostka> tip </ciekawostka> LF]]
    foreach param city/param [append parametry rejoin [form <parametr> param </parametr> LF]]

    if all [not force exists? file: to-file file] [return none]
    attempt [
      write file rejoin [
       {<?xml version="1.0" encoding="} rebol-encoding {"?>} LF
        <miejscowosc> LF 
          <nazwa> LF
            <aktualna> form first city/name </aktualna> LF
            <historyczna> form second city/name </historyczna> LF
          </nazwa> LF
           LF
          <lokalizacja> LF
            <opis> form city/direction </opis> LF
            <geograficzna> LF
              <dlugosc> form first city/gps </dlugosc> LF
              <szerokosc> form second city/gps </szerokosc> LF
              <wysokosc> form third city/gps </wysokosc> LF
            </geograficzna> LF
          </lokalizacja> LF
           LF
          <informacje> LF
            <opis> form city/info </opis> LF
             LF
            <ciekawostki> LF ciekawostki </ciekawostki> LF
             LF
            <parametry> LF parametry </parametry> LF
             LF
            <autor> LF
              <osoba> author </osoba> LF
              <email> form email </email> LF
              <data> form now/date </data> LF
              <copyleft> {(C)2001-2005 OSR http://www.rowery.olsztyn.pl/} </copyleft> LF
            </autor> LF
           LF
          </informacje> LF
        </miejscowosc> LF
      ]
      true
    ]
  ]

  preload-xml: func [
   "Wczytuje plik XML i konwertuje znaki; Zwraca object! lub none!"
    file [string! file!] "nazwa pliku"
   /local xml doc obj
  ][

    from-encoding: none
    to-encoding: rebol-encoding

    attempt [
      xml: read to-file file

      parse/all xml [thru "<?xml" thru {encoding="} copy from-encoding to {"} thru "?>" break]
      if any [none? from-encoding empty? trim from-encoding] [from-encoding: rebol-encoding]

      doc: xml-to-object parse-xml+ (plchars xml from-encoding to-encoding)
      obj: make object! doc
    ]
  ]

  set 'osr-load func [
   "Odczytuje informacje o miejscowosci z pliku XML (wymaga xml-parse i xml-object); Zwraca city-object! lub none!"
    file [string! file!] "nazwa pliku"
   /local obj tip param from-encoding to-encoding tmp
  ][
    
    if none? (obj: preload-xml file) [return none]

    tip: copy []
    param: copy []

    attempt [
      make city-object compose/deep [
        name: [
          (obj/document/miejscowosc/nazwa/aktualna/value?) 
          (either tmp: attempt [copy obj/document/miejscowosc/nazwa/historyczna/value?] [tmp] [{}])
        ]
        direction: (obj/document/miejscowosc/lokalizacja/opis/value?)
      ; gps: [0 0 0]
        info: (obj/document/miejscowosc/informacje/opis/value?)
        tip: [(foreach item to-block obj/document/miejscowosc/informacje/ciekawostki/ciekawostka [append tip item/value?])]
        param: [(foreach item to-block obj/document/miejscowosc/informacje/parametry/parametr [append param item/value?])]
      ]
    ]
  ]

  set 'param-load func [
   "Odczytuje parametry miejscowosci z pliku XML (wymaga xml-parse i xml-object); Zwraca block! lub none!"
    file [string! file!] "nazwa pliku"
   /local obj param from-encoding to-encoding
  ][

    if none? (obj: preload-xml file) [return none]
    param: copy []

    attempt [
      compose/deep [(foreach item to-block obj/document/parametry/parametr [append param item/value?])]
    ]
  ]
]
