REBOL [
  Title:  "OSR/miasta" 
  Version: 1.3.1
  Date:    2005-02-25
  Author: "Piotr Gapinski"
  Email:  {news [at] rowery! olsztyn.pl}
  File:   %osr-gui.r
  Info: {
    Program upraszczajacy zbieranie informacji o miastach (na potrzeby OSR).
    Olsztynska Strona Rowerowa http://www.rowery.olsztyn.pl/
    PROGRAM NA LICENCJI GPL (w wersji II)
  }
]

osr-gui-ctx: context [
  do %osr-file.r
  do %osr-encoding.r
  do %osr-style.r
  ; do %area-scroll-style.r

  osr-feel: stylize/styles [
    styles 
    b-city:  btn-osr "Miejscowosc"    [panels/pane: p-city show panels]
    b-info:  btn-osr "Opis"           [panels/pane: p-info show panels]
    b-tip:   btn-osr "Ciekawostki"    [panels/pane: p-tip show panels]
    b-param: btn-osr "Parametry"      [panels/pane: p-param show panels]
    b-load:  btn-osr "Wczytaj..."     [load-file]
    b-clear: btn-osr "Wyczysc"        [if (request/offset/confirm "Wyczyscic wszystkie dane?" (window/offset + (window/size / 4))) [clear-gui]]
    b-save:  btn-osr "Zapisz jako..." [save-file]
    b-about: btn-osr "O programie"    [panels/pane: p-about show panels] 
    b-quit:  btn-red "Koniec"         [quit]
  ] osr-look
  set 'osr-look-feel osr-feel

  p-city: layout [
    ; panel opisujacy miejsce (nazwa, nazwa_historyczna, polozenie)
    ;
    styles osr-look
    origin 2x2 space 2x2 across
    lab "Nazwa miejscowosci" return 
    here: at
    fc-name: field (450 - here/x) font-name font-fixed return
    lab "Nazwa historyczna" return
    fc-hist: field (450 - here/x) font-name font-fixed return
    lab "Polozenie wzgledem wiekszych miast" return
    here: at
    fc-loc: area (450x300 - here) wrap font-name font-fixed return
  ]

  p-info: layout [
    ; panel opisu miejscowosci
    ;
    ; styles area-style
    styles osr-look
    origin 2x2 space 2x2 label "Szczegolowy opis miejscowosci"
    guide here: at
    fi-info: area (450x300 - 15x0 - 4x0 - here) wrap font-name font-fixed return
    fi-sld: scroller (as-pair 15 (300 - here/y)) [svt fi-info fi-sld]
    ; fi-info: area-scroll (450x300 - 0x4 - here) wrap font-name font-fixed vscroll 15
  ]

  p-tip: layout [
    ; panel ciekawostek
    ;
    ; styles area-style
    styles osr-look
    origin 2x2 space 2x2 
    label {Ciekawostki i slawni mieszkancy (kazdy wpis w oddzielnej linii)}
    here: at
    ft-tip: area (450x300 - 0x15 - 0x4 - here) font-name font-fixed
    ft-sld: scroller (as-pair (450 - here/x) 15) [sht ft-tip ft-sld]
    ; ft-tip: area-scroll (450x300 - 0x4 - here) font-name font-fixed hscroll 15
  ]
  
  if none? (param: param-load %osr-param.r) [alert "Brak pliku parametrow!"]
  p-param: layout [
    ; panel opisujacy parametry
    ;
    styles osr-look
    origin 2x2 space 2x2 lab {Parametry zwiazane z miejscowoscia} guide
    here: at
    fp-from: text-list (as-pair 205 (300 - here/y)) data sort param
    return  
    btn 30 "=>" [add-param]    ; dodaj parametr
    btn 30 "<=" [remove-param] ; usun parametr
    btn 30 "!"  [clear-param]  ; usun wszystkie parametry
    return
    fp-to: text-list (as-pair 205 (300 - here/y))
    return
  ]

  p-about: layout [
    ; panel informacji o programie
    ;
    styles osr-look
    origin 2x2 space 2x2 across
    here: at
    txt (450 - here/x) {
      Program OSR/miasta powstawal w latach 2001-2005 jako narzedzie do pomocy przy
      wprowadzaniu opisow miejscowosci. Przygotowane w ten sposob materialy sa 
      publikowane pod adresem http://www.rowery.olsztyn/wiki/miejsca/} return
    txt {(C) Piotr Gapinski} return
    txt {Program na licencji GNU GPL} return
    pad 0x10
    btn "Olsztynska Strona Rowerowa" [browse http://www.rowery.olsztyn.pl] return
    btn "Rebol.com" [browse http://www.rebol.com] 
    btn "Rebol.net" [browse http://www.rebol.net]
    btn "Rebol.org" [browse http://www.rebol.org] return
  ]

  p-city/offset:  0x0
  p-info/offset:  0x0
  p-tip/offset:   0x0
  p-param/offset: 0x0
  p-about/offset: 0x0

  set 'gui-panels reduce [p-city p-info p-tip p-param p-about]

  export-city: does [
    ; tworzy city-object na podstawie danych pobranych bezposrednio z pol formularzy
    ;
    attempt [
      make city-object [
        name: reduce [fc-name/text fc-hist/text] ; [aktualna historyczna]
        direction: fc-loc/text                   ; [polozenie]
        info: fi-info/text                       ; [opis]
        tip: parse/all trim ft-tip/text "^/"     ; [tip tip tip]
        param: to-block fp-to/data               ; [param param param]
      ]
    ]
  ]

  import-city: func [
   "Przenosi dane z city-object do pol formularzy programu; zwraca true!"
    city [object!] "obiekt opisu miejscowosci"
  ][
    attempt [
      fc-name/text: city/name/1                  ; aktualna
      fc-hist/text: city/name/2                  ; historyczna
      fc-loc/text:  city/direction               ; polozenie
      fi-info/text: city/info                    ; opis
      foreach item  city/tip [append ft-tip/text rejoin [item "^/"]] ; ciekawostki rozdzielone "enterem"
      foreach item  city/param [attempt[move-param item]]
      true
    ]
  ]

  load-file: has [file city] [
    ; odczyt danych z pliku .osr
    ;
    any [
      none? file: request-file/title/filter/keep/only "Wczytaj plik" "Load" "*.osr"

      ; wyswietl informacje jezeli plik nie jest prawidlowy
      ;
      ;;none? city: osr-load file
      either (none? city: osr-load file) [request/offset/ok "Nieprawidlowy plik danych!" (window/offset + (window/size / 4))] [false]

      attempt [
        ; nie zapomniej o wyczyszczeniu formularzy przed zaladowaniem danych!
        ;
        clear-gui 
        import-city city 
        refresh-gui
      ]
    ]
  ]

  save-file: has [file-name file city] [
    ; zapis pol formularzy do pliku .osr
    ;
    city: export-city

    ; sugeruj nazwe zgodna z szablonem [nazwa "_" historyczna ".osr"]
    ;
    file-name: either (empty? trim city/name/2) [city/name/1] [rejoin [city/name/1 "_" city/name/2]]
    append file-name ".osr"

    any [
      none? file: request-file/title/filter/file/keep/only "Zapisz plik" "Save" "*.osr" file-name

      ; nadpisywanie pliku?
      ;
      ;;not (either (exists? file) [request/offset/confirm "Nadpisac plik?" (window/offset + (window/size / 4))] [true])
      either (exists? file) [not request/offset/confirm "Nadpisac plik?" (window/offset + (window/size / 4))] [false]

      osr-save/force city file user-name user-email ; informacje o userze sa globalne!
    ]
  ]

  clear-gui: does [
    clear-fields p-city
    clear fc-loc/text fc-loc/line-list: none
    clear-fields p-info
    clear fi-info/text fi-info/line-list: none
    clear-fields p-tip
    clear ft-tip/text ft-tip/line-list: none
    clear-param
    refresh-gui
  ]
  
  refresh-gui: does [show gui-panels] ;; foreach panel gui-panels [show panel]]
  
  add-param: has [param-pos list-size list-pos] [
    ; dodaje parametr do listy parametrow miejscowosci
    ; jezeli nic nie jest zaznaczone - nie rob nic!
    ;
    if (empty? fp-from/picked) [return]

    ; znajdz parametr z pliku i usun odpowiadajacy mu parametr z listy zrodlowej
    ; zakladamy, e skoro jest to zaznaczony element w liscie - musi istniec taki parametr 
    ;
    param-pos: find fp-from/data fp-from/picked
    append fp-to/data fp-from/picked
    remove param-pos
    list-size: length? fp-from/data

    ; po usunieciu parametru z listy jaka jest jej dlugosc?
    ;
    either (list-size > 0) [
      list-pos: index? param-pos
      fp-from/picked: copy either ((list-size - list-pos) > 0) [

        ; jezeli cos pozostalo po usunieciu parametru, zaznacz jego nastepce
        ;
        first param-pos
      ][
        ; w przeciwnym razie zaznacz ostatni element
        ;
        last fp-from/data
      ]
      list-pos: list-pos - 1
    ][
      ; lista jest pusta, skasuj zaznaczony wczesniej element do przeniesienia 
      ;
      list-pos: 0
      fp-from/picked: copy []
    ]

    ; uaktualnij pozycje od ktorej rozpoczyna sie wyswietlanie listy
    ;
    update-param
  ]

  move-param: func [
   "Przenosi parametr do listy parametrow miejscowosci"
    param [string!] 
   /local param-pos rc
  ][
    ; sprawdz czy taki parametr istnieje na liscie
    ;
    param-pos: find fp-from/data param
    either not none? param-pos [
      ; istnieje, zaznacz go jako aktywny (do przeniesienia)
      ;
      fp-from/picked: copy param
      add-param
    ][
      ; parametr taki nie istnieje
      ; sprawdz "najblizszy odpowiednik"
      ;
      rc: none
      foreach i fp-from/data [
        rc: find i param
        if not none? rc [find fp-from/data rc break]
      ]
      if not none? rc [
        fp-from/picked: copy rc
        add-param
      ]
    ]
  ]

  remove-param: has [rem-picked] [
    ; usuwa parametr z listy parametrow miejscowosci
    ; nie uaktywania kolejnego parametru do usuniecia (kolejny element trzeba wskazac samodzielnie)
    ;
    rem-picked: fp-to/picked
    append fp-from/data rem-picked

    ; w liscie zrodlowej ustaw przenoszony element jako aktywny
    ;
    if (not empty? rem-picked) [fp-from/picked: copy rem-picked]
    sort fp-from/data
    remove find fp-to/data rem-picked
    fp-to/picked: copy []
    update-param
  ]

  clear-param: does [
    ; usuwa wszystkie parametry z listy parametrow miejscowosci
    ; w liscie zrodlowej ustawia aktywny parametr na pierwszy element listy
    ;
    append fp-from/data fp-to/data
    fp-to/line-list: none
    clear fp-to/data
    if (not empty? fp-from/data) [fp-from/picked: copy first fp-from/data]
    sort fp-from/data
    update-param
  ]

  update-param: does [
    ; odswieza wyglad list (wlacznie z rekalkulacja wielkosci scrolerow)
    ;
    fp-from/sld/redrag fp-from/lc / max 1 length? head fp-from/lines ;resets slider size
    if (empty? fp-to/data) [fp-to/sld/data: 0]

    fp-to/sld/redrag fp-to/lc / max 1 length? head fp-to/lines ;resets slider size
    fp-to/sld/data: fp-from/sld/data: 0
    show p-param
  ]

  svt: func [tf sf /local tmp][  ;!!! fix bug in View! (not - 0x30)
    if none? tf/para [exit]
    tmp: min 0x0 tf/size - (size-text tf) - 10
    either sf/size/x > sf/size/y [tf/para/scroll/x: sf/data * first tmp] [tf/para/scroll/y: sf/data * second tmp]
    show [tf sf]
  ]

  sht: func [tf sf /local tmp][  ;!!! fix bug in View! (not - 0x30)
    if none? tf/para [exit]
    tmp: min 0x0 tf/size - (size-text tf) - 10
    either sf/size/x > sf/size/y [tf/para/scroll/x: sf/data * first tmp] [tf/para/scroll/y: sf/data * second tmp]
    show tf
  ]
]